#pragma once

#include <boost/property_map.hpp>

// Class for edge length computation operator (A&B)
class Edge_length_func
{
public:

  // Boost property type definitions:
  typedef boost::readable_property_map_tag        category;
  typedef double                                  value_type;
  typedef value_type                              reference;
  typedef Arrangement_2::Halfedge_handle          key_type;

  double operator() (Arrangement_2::Halfedge_handle e) const
  {
    const double     x1 = CGAL::to_double (e->source()->point().x());
    const double     y1 = CGAL::to_double (e->source()->point().y());
    const double     x2 = CGAL::to_double (e->target()->point().x());
    const double     y2 = CGAL::to_double (e->target()->point().y());
    const double     diff_x = x2 - x1;
    const double     diff_y = y2 - y1;

    return (std::sqrt (diff_x*diff_x + diff_y*diff_y));
  }
};

// get edge length for boost's dijkstra
double get(Edge_length_func edge_length, Arrangement_2::Halfedge_handle e) 
{ 
  return (edge_length (e));
}
