#pragma once

#include "typedef.h"
#include "Problem_input.h"
#include "Problem_info.h"

// Forward declaration of Problem_input.
class Problem_input;

/*******************************************************************************************
 * The class containes the main algorithm - the Path_planner_algorithm
 *******************************************************************************************/

class Path_planner_algorithm
{

public:
	
	// Constructor - receives the input of the problem.
	Path_planner_algorithm(Problem_input *input);
	
	// Distructor
	virtual ~Path_planner_algorithm(void);
	
	// The function runs the algorithm.
	Problem_info run_algorithm();

protected:
	
	
	//Declarations of halfedge_set (need to define an "<" operator) 
	template <class Handle> struct Less_handle
	{
		bool operator() (Handle h1, Handle h2) const 
		{
			return (&(*h1) < &(*h2));
		}
	};

	typedef std::set<Halfedge_handle, Less_handle<Halfedge_handle> >  Halfedge_set;
	
	// Construct the vertical decomposition of the given arrangement.
	void vertical_decomposition(Arrangement_2& arr, Halfedge_set& v_edges);
	
	// Add a vertical segment from the given vertex to some other arrangement feature. (for the verticl decomposition)
	Halfedge_handle add_vertical_segment(Arrangement_2& arr, Vertex_handle v, CGAL::Object obj);
	
	// Calculate the middle point of an edge.
	Point_2 calculateEdgeMidPoint(Arrangement_2::Halfedge& he);

	Face_const_handle locateFace(Arrangement_2 & arr,Point_2 & p);
	
	//This function finds a path between source and target
	Polyline_2 extractPath(Arrangement_2 & arr,Point_2 & source,Point_2 & target);

	Face_handle Face_handle_unconst(const Face_const_handle fc);
	
	//This function calculates the center of a face.
	Point_2 calculateCenterOfFace(Arrangement_2::Face& f);

	// The created info
	Problem_info m_info;

	// The problem input
	Problem_input *m_input;
};