#include "stdafx.h"
#include "Polygon_drawer.h"
#include "Polyline_drawer.h"
#include "Path_viewer.h"
#include "typedef.h"
#include "Problem_info.h"
#include "Operators.h"
#include <CGAL/Bbox_2.h>
#include <glut.h>
#include <boost/lexical_cast.hpp>
#include <stdlib.h>

// initializing statics.
Problem_info * Path_viewer::m_info = NULL;
Bbox_2 Path_viewer::m_bbox;
std::list<Polygon_drawer> Path_viewer::m_polygons_drawers;

// initial width and height of the window.
const int WIDTH = 720;
const int HEIGHT = 576;


Path_viewer::Path_viewer(Problem_info *problem_info)
{
	m_info = problem_info;
}

Path_viewer::~Path_viewer(void)
{
}

// Shows the marker on the screen.
void Path_viewer::show(int argc, char * argv[])
{
	glutInit(&argc, argv);
    glutInitDisplayMode(GLUT_DOUBLE | GLUT_RGB);
    glutInitWindowPosition(50, 50);

    glutInitWindowSize(WIDTH, HEIGHT);
    glutCreateWindow("Polygon path example");

	m_bbox = calculate_bbox();

	glutDisplayFunc(Path_viewer::draw);
	glutReshapeFunc(Path_viewer::reshape);
    glutMainLoop();
}

// Draw call-back from GLUT.
void Path_viewer::draw()
{		
	glClear(GL_COLOR_BUFFER_BIT);
	
	// Draw the polygons of the obstacles
    std::list<Polygon_drawer>::iterator it;
    for (it= m_polygons_drawers.begin(); it!= m_polygons_drawers.end(); it++)
    {
        it->draw();
    }
	
	// Draw the path
	Polyline_drawer path_drawer = Polyline_drawer(m_info->path(), 0, 0.5, 1);
    path_drawer.draw();

	// Draw the target
	Polyline_2 target_point;
	target_point.push_back(m_info->input()->source());
	target_point.push_back(m_info->input()->target());
	Polyline_drawer target_drawer = Polyline_drawer(target_point, 1, 0, 0);
	target_drawer.draw();

	glutSwapBuffers();
}

// Reshape call-back from GLUT.
void Path_viewer::reshape(int width, int height)
{
    double w_width = m_bbox.xmax() - m_bbox.xmin();
    double w_height = m_bbox.ymax() - m_bbox.ymin();

    glClearColor(0.0, 0.0, 0.0, 0.0);
    glColor3f(0.0, 0.0, 1.0);
    glMatrixMode(GL_PROJECTION);	
    glLoadIdentity();
    glOrtho( m_bbox.xmin() - 0.1*w_width, 
             m_bbox.xmin() + 1.1*w_width, 
             m_bbox.ymin() - 0.1*w_height, 
             m_bbox.ymin() + 1.25*w_height,
             -1, 1);

	glViewport(0, 0, width, height);
}

// copies the information needed for drawing the configuration space in glut. Returns the bounding box of
// all curves.
Bbox_2 Path_viewer::calculate_bbox()
{
    Polygon_2 bbox;
	std::list<Polygon_drawer> polygons_drawers;
    for (Polygons::const_iterator it= m_info->input()->obstacles().begin();
         it!= m_info->input()->obstacles().end();
         it++)
    {
        const Polygon_2& pol = *it;
        bbox.insert(bbox.vertices_end(), pol.vertices_begin(), pol.vertices_end());
        polygons_drawers.push_back(Polygon_drawer(Polygon_with_holes_2(pol), 0.0f, 0.0f, 1.0f));
    }
    
    Vector_2 vec = m_info->input()->source() - CGAL::ORIGIN;
    Polygon_2 moved_robot = m_info->input()->robot() + vec;
    bbox.insert(bbox.vertices_end(), moved_robot.vertices_begin(), moved_robot.vertices_end());
    polygons_drawers.push_back(Polygon_drawer(Polygon_with_holes_2(moved_robot), 1.0, 0.0, 0.0));
	
	Polygons_wh temp_c_obstacles = m_info->c_obstacles(); 
    Polygons_wh::const_iterator c_it;
    for (c_it = temp_c_obstacles.begin(); 
         c_it != temp_c_obstacles.end(); 
         c_it++)
    {
        bbox.insert(bbox.vertices_end(), c_it->outer_boundary().vertices_begin(), c_it->outer_boundary().vertices_end());
        polygons_drawers.push_front(Polygon_drawer(*c_it, 0.0, 1.0, 0.0));
    }
	
	Polygons temp_c_room = m_info->c_room();
    for (Polygons::const_iterator it = temp_c_room.begin(); 
         it != temp_c_room.end(); 
         it++)
    {
		bbox.insert(bbox.vertices_end(), it->vertices_begin(), it->vertices_end());
        polygons_drawers.push_front(Polygon_drawer(Polygon_with_holes_2(*it), 1.0, 1.0, 0.0));
    }
	
	m_polygons_drawers = polygons_drawers;
    
	return bbox.bbox();
}



