
#include "stdafx.h"
#include "typedef.h"
#include "Problem_input.h"
#include "Problem_input_reader.h"
#include "Path_viewer.h"
#include "Operators.h"
#include "Path_planner_algorithm.h"
#include <stdlib.h>
#include <CGAL/Random.h>
#include <boost/program_options.hpp>
#include <boost/program_options/parsers.hpp>
#include <boost/program_options/options_description.hpp>
#include <boost/program_options/variables_map.hpp>

namespace po = boost::program_options;

using namespace std;

/*******************************************************************************************
 * PolygonPathExample.cpp : Defines the entry point for the console application.
 *******************************************************************************************/

int main(int argc, char * argv[])
{
	
	// Use of Boost Program options to pares command line easily.
    po::options_description desc("Allowed options");
    desc.add_options()
        ("help", "produce help message")
        ("input-file,I", po::value< string >(), "input file")
        ;
    
    po::variables_map vm;

	// Parse the caommand line
    po::store(po::parse_command_line(argc, argv, desc), vm);
    po::notify(vm);    

    if (vm.count("help")) 
    {
        cout << "The input for the program will be as follows: " << endl;
        cout << "   0.  The room as polygon." << endl;
        cout << "   1.  The obstacles - first the number of the obstacles and then the obstacles themselves (which are sets of points which we compute their convex hull)." << endl;
        cout << "   2.  The robot - which is a polygon." << endl;
        cout << "   3.  The source point - where the robot is initially located." << endl;
        cout << "   4.  The target of the robot." << endl;

        cout << desc << "\n";
        return 1;
    }

    if (vm.count("input-file") == 0) 
    {
        std::cout << "Input file was not specified!" << std::endl;
		return 1;
    }

    // Read the input
	Problem_input_reader problem_input_reader(vm["input-file"].as< string >()); 
    Problem_input input = problem_input_reader.read();
	
	// Initialize the algorithm.
	Path_planner_algorithm alg(&input);

	// Solve the problem
	Problem_info problem_info= alg.run_algorithm();

	// view the result.
	Path_viewer viewer(&problem_info);
	viewer.show(argc, argv);

	return 0;

} 