#pragma once

#include "Path_viewer.h"
#include "typedef.h"
#include <glut.h>
#include <list>
#include <CGAL/basic.h>
#include <CGAL/Partition_traits_2.h>
#include <CGAL/partition_2.h>

// This class draws a polygon.
class Polygon_drawer
{
  public:

    Polygon_drawer(const Polygon_with_holes_2& poly, float red, float green, float blue, 
                   float hole_red=1, float hole_green=1, float hole_blue=0) 
        : _red(red), _green(green), _blue(blue), 
		_hole_red(red), _hole_green(green), _hole_blue(blue)
	{
		// partition the polygon into convex polygons.
		Polygon_2 temp = poly.outer_boundary();
		if (temp.is_counterclockwise_oriented()==false)
			temp.reverse_orientation();
		CGAL::approx_convex_partition_2(temp.vertices_begin(), temp.vertices_end(), 
			std::back_inserter(_polygons));

		// partition each of the holes into convex polygons.
		Polygon_with_holes_2::Hole_const_iterator it;
		for (it=poly.holes_begin(); it!=poly.holes_end(); it++)
		{
			temp = *it;
			if (temp.is_counterclockwise_oriented()==false)
				temp.reverse_orientation();
			CGAL::approx_convex_partition_2(temp.vertices_begin(), temp.vertices_end(), 
				std::back_inserter(_holes));
		}
	}
	
	// a draw function that calls the next draw function with the right parameters.
	void draw();
	
	template <typename ForwardIterator>
	void draw(ForwardIterator begin, ForwardIterator end, GLenum mode, float red, float green, float blue);
 
private:

	Partition_polygon_list _polygons;
	Partition_polygon_list _holes;

	float _red;
    float _green;
    float _blue;
    float _hole_red;
    float _hole_green;
    float _hole_blue;
};