#include "stdafx.h"
#include "Problem_input_reader.h"
#include "Problem_input.h"
#include "typedef.h"
#include "Operators.h"
#include <iostream>
#include <fstream>
#include <CGAL/IO/Arr_iostream.h>
#include <boost/lexical_cast.hpp>
#include <CGAL/ch_graham_andrew.h>

Problem_input_reader::Problem_input_reader(const std::string &file_name)
: m_file_name(file_name)
{
}

Problem_input_reader::~Problem_input_reader(void)
{
}

// Reads the input file.
// Return value: the problem input from the input file.
Problem_input Problem_input_reader::read()
{

	Problem_input input;
	std::ifstream ifile(m_file_name.c_str());
	std::istream *in = &ifile;

	// 1) Room.
	Polygon_2 room;
    *in >> room;
	input.set_room(room);
	
	int num_obstacles;
	*in >> num_obstacles;

	// 2) Obstacles.
	Polygons obstacles;
	for (int i = 0; i < num_obstacles; i++)
    {
		// Create the current polygon.
        Polygon_2 cur;
        *in >> cur;
        
		// Computes the convex hull of the points and add it to the list. 
        Polygon_2 convex;
        CGAL::ch_graham_andrew(cur.vertices_begin(), cur.vertices_end(), back_inserter(convex));
        if (convex.is_counterclockwise_oriented()==false)
            convex.reverse_orientation();
        obstacles.push_back(convex);
    }

	input.set_obstacles(obstacles);
	
	// 3) Robot.
	Polygon_2 robot;
	*in >> robot;
    input.set_robot(robot);
	
	// 4) Source point.
	Point_2 source;
    *in >> source;
	input.set_source(source);

	// 5) Target point.
	Point_2 target;
    *in >> target;
	input.set_target(target);
	
	return input;
}
